import React, { Component } from "react"
import "./index.scss"
import Container from "../container"
import classnames from "classnames"
import { html, isValidHttpUrl } from "src/utils"

class SingleAnswerQuestion extends Component {
  icon =
    "https://julyedu-cdn.oss-cn-beijing.aliyuncs.com/interactive_tutorial/study/single-answer-icon.png"

  state = {
    correct: "",
    wrong: "",
    replies: "",
    selectable: false,
  }

  handleSelect = (info, index) => {
    if (!this.state.selectable) {
      return
    }

    const {
      question: { unit_questions, syllabus_id },
      updatePosition,
      resetStatus,
      saveSchedule,
      addCache,
      savePractice,
    } = this.props
    const userSelect = unit_questions[index]
    const isCorrect = userSelect.is_ans
    this.setState(
      {
        wrong: isCorrect ? "" : index,
        correct: isCorrect ? index : "",
        replies: info.slice(0, 1),
        selectable: false,
      },
      () => {
        updatePosition()
        resetStatus()
        addCache(info.slice(1))
        savePractice({
          type: 1,
          syllabus_id: syllabus_id,
          answer: unit_questions[index].unit_id,
          result: isCorrect,
        })
        saveSchedule(syllabus_id, info[0].info_id)
      }
    )
  }

  componentDidMount() {
    const {
      user_answer: userAnswer,
      unit_questions: options,
    } = this.props.question
    if (userAnswer && userAnswer.answer) {
      const userSelectId = userAnswer.answer
      // eslint-disable-next-line eqeqeq
      const userSelectIndex = options.findIndex(
        (item) => item.unit_id == userSelectId
      )
      const isUserCorrect = options[userSelectIndex].is_ans
      this.setState({
        correct: isUserCorrect ? userSelectIndex : "",
        wrong: isUserCorrect ? "" : userSelectIndex,
        replies: options[userSelectIndex].unit_info,
        selectable: false,
      })
    } else {
      this.setState({
        selectable: true,
      })
    }
  }

  render() {
    const { correct, wrong, replies } = this.state
    const {
      topic,
      img,
      question: { unit_questions: options },
      updatePosition,
    } = this.props
    return (
      <>
        <Container
          user={this.icon}
          content={
            <div className={"single-answer-question"}>
              <div
                className="topic"
                dangerouslySetInnerHTML={html(topic)}
              ></div>
              {img && <img src={img} alt="" />}
              <ul>
                {options &&
                  !!options.length &&
                  options.map((item, index) => {
                    return (
                      <li
                        className={classnames([
                          "option",
                          {
                            correct: index === correct,
                            wrong: index === wrong,
                          },
                        ])}
                        key={index}
                        onClick={this.handleSelect.bind(
                          this,
                          item.unit_info,
                          index
                        )}
                      >
                        {item.des}
                      </li>
                    )
                  })}
              </ul>
            </div>
          }
        />
        {!!replies.length &&
          replies.map((item) => {
            if (isValidHttpUrl(item.content)) {
              return (
                <Container key={item.info_id}>
                  <img src={item.content} alt="" onLoad={updatePosition} />
                </Container>
              )
            }
            return <Container content={item.content} key={item.info_id} />
          })}
      </>
    )
  }
}

export default SingleAnswerQuestion
