import React, { Component } from "react"
import "./index.scss"
import { Terminal } from "xterm"
import "xterm/css/xterm.css"
import { FitAddon } from "xterm-addon-fit"
import { Toast } from "antd-mobile"
import axios from "axios"

interface Props {
  result: string
  executing: boolean
  filename: string
}

class TerminalInterface extends Component<Props> {
  terminal: Terminal | undefined
  termElem: HTMLElement | null = null
  quit = "\u0003"
  socket: WebSocket | null = null
  endFlag: string = "wrvgh38v3hc923hcv39sa"

  state = {
    terminalWidth: "",
    userInput: "",
    isEnding: false,
  }

  componentDidMount() {
    const content = document.querySelector(".container .content")
    const contentStyles = window.getComputedStyle(content!)
    const contentWidth = content!.clientWidth
    const contentPaddingLeft = parseFloat(
      contentStyles.getPropertyValue("padding-left")
    )
    this.setState({
      terminalWidth: `${contentWidth - contentPaddingLeft}px`,
    })

    this.terminal = new Terminal()
    const fitAddon = new FitAddon()
    this.terminal.loadAddon(fitAddon)
    this.terminal.open(this.termElem!)

    const { result, executing } = this.props
    setTimeout(() => {
      fitAddon.fit()
      if (result) {
        this.setState({
          isEnding: true,
        })
        this.terminal && this.terminal.write(this.props.result)
      } else if (executing) {
        this.getAuthenticationData()
      }
    })
  }

  getAuthenticationData = () => {
    axios
      .post(
        // `http://47.93.119.175:8888/auth`,
        `http://webssh-test.julyedu.com:8888/auth`,
        {
          username: "3guh394h3fhj0f4",
          password: "okqdw029j038hrv3890cv",
        },
        {
          withCredentials: false,
        }
      )
      .then((res) => {
        const { code, msg, data } = res.data
        if (code === 0) {
          this.getLinkId(data)
        } else {
          Toast.fail(msg, 2)
        }
      })
  }

  getLinkId = (token: string) => {
    axios
      .post(
        `http://webssh-test.julyedu.com:8888`,
        {},
        {
          headers: {
            Token: token,
          },
          withCredentials: false,
        }
      )
      .then((res) => {
        const { id } = res.data
        this.connectServer(id)
      })
  }

  connectServer = (id: string) => {
    this.socket = new WebSocket(`${API.ws}?id=${id}`)
    this.socket.onopen = () => {
      this.sendMessage(this.props.filename)
      this.sendMessage("\r")
    }
    this.socket.onmessage = (event) => {
      const reader = new FileReader()
      reader.onload = () => {
        if (this.ab2str(reader.result).trim() === this.endFlag) {
          return this.socket?.close()
        }
        this.terminal!.write(this.ab2str(reader.result))
      }
      reader.readAsArrayBuffer(event.data)
    }
    this.socket.onerror = (event) => {
      console.log(event)
      // this.connectServer(id)
    }
    this.socket.onclose = (event) => {
      console.log("closed")
    }
  }

  ab2str = (buf: any) => {
    return String.fromCharCode.apply(null, new Uint8Array(buf) as any)
  }

  handleChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    this.setState({
      userInput: e.target.value,
    })
  }

  sendMessage = (message: string) => {
    this.socket?.send(
      JSON.stringify({
        data: message,
      })
    )
  }

  render() {
    const { terminalWidth, isEnding } = this.state
    return (
      <div className={"terminal-container"}>
        <div
          className={"terminal"}
          ref={(el) => (this.termElem = el)}
          style={{ width: `${terminalWidth}` }}
        ></div>
        {!isEnding && (
          <div className="input">
            <input
              type="text"
              id={"terminal-input"}
              placeholder={"输入完毕后请点击确认"}
              onChange={this.handleChange}
            />
            <label htmlFor="terminal-input">
              <button
                onClick={() => {
                  this.sendMessage(this.state.userInput)
                }}
              >
                确认
              </button>
            </label>
          </div>
        )}
      </div>
    )
  }
}

export default TerminalInterface
